__all__ = ('LoanIssuePoint', 'LoanPaymentBodyPoint', 'LoanInterestsChargePoint', 'LoanPaymentInterestsPoint',
           'RewardChargePoint',
           )

from expressmoney.api import *
from .balances import *

SERVICE = 'accounting'
APP = 'operations'


class OperationContract(Contract):
    amount = serializers.DecimalField(max_digits=16, decimal_places=0, min_value=1, max_value=1000000000)
    is_reverse = serializers.BooleanField(default=False)
    department = serializers.IntegerField(min_value=1)
    analytical = serializers.IntegerField(min_value=1)


class LoanIssueCreateContract(OperationContract):
    pass


class LoanPaymentBodyCreateContract(OperationContract):
    pass


class LoanInterestsChargeCreateContract(OperationContract):
    pass


class LoanPaymentInterestsCreateContract(OperationContract):
    pass


class RewardChargeCreateContract(OperationContract):
    amount = serializers.DecimalField(max_digits=16, decimal_places=0, min_value=1, max_value=1000000000)
    is_reverse = serializers.BooleanField()


class LoanIssueID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'loan_issue'


class LoanPaymentBodyID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'loan_payment_body'


class LoanInterestsChargeID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'loan_interests_charge'


class LoanPaymentInterestsID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'loan_payment_interests'


class RewardChargeID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'reward_charge'


class LoanIssuePoint(CreatePointMixin, ContractPoint):
    _point_id = LoanIssueID()
    _create_contract = LoanIssueCreateContract

    def _get_related_points(self) -> list:
        related_points = super()._get_related_points()
        loan = self._payload.get('analytical')
        related_points.append(A48801BalanceAnalyticalPoint(self._user, loan))
        return related_points


class LoanPaymentBodyPoint(CreatePointMixin, ContractPoint):
    _point_id = LoanPaymentBodyID()
    _create_contract = LoanPaymentBodyCreateContract

    def _get_related_points(self) -> list:
        related_points = super()._get_related_points()
        loan = self._payload.get('analytical')
        related_points.append(A48801BalanceAnalyticalPoint(self._user, loan))
        return related_points


class LoanInterestsChargePoint(CreatePointMixin, ContractPoint):
    _point_id = LoanInterestsChargeID()
    _create_contract = LoanInterestsChargeCreateContract

    def _get_related_points(self) -> list:
        related_points = super()._get_related_points()
        loan = self._payload.get('analytical')
        related_points.append(A48802BalanceAnalyticalPoint(self._user, loan))
        return related_points


class LoanPaymentInterestsPoint(CreatePointMixin, ContractPoint):
    _point_id = LoanPaymentInterestsID()
    _create_contract = LoanPaymentInterestsCreateContract

    def _get_related_points(self) -> list:
        related_points = super()._get_related_points()
        loan = self._payload.get('analytical')
        related_points.append(P48809BalanceAnalyticalPoint(self._user, loan))
        return related_points


class RewardChargePoint(CreatePointMixin, ContractPoint):
    _point_id = RewardChargeID()
    _create_contract = RewardChargeCreateContract



__all__ = ('A48801BalanceAnalyticalPoint', 'A48802BalanceAnalyticalPoint', 'P48809BalanceAnalyticalPoint',
           'P47422BalanceAnalyticalObjectPoint',
           )

from expressmoney.api import *

SERVICE = 'accounting'
APP = 'balances'


class A48801BalanceAnalyticalReadContract(Contract):
    id = serializers.IntegerField(min_value=1)
    updated = serializers.DateTimeField()
    balance = serializers.DecimalField(max_digits=16, decimal_places=0)


class A48802BalanceAnalyticalReadContract(A48801BalanceAnalyticalReadContract):
    pass


class P48809BalanceAnalyticalReadContract(A48801BalanceAnalyticalReadContract):
    pass


class P47422BalanceAnalyticalReadContract(A48801BalanceAnalyticalReadContract):
    pass


class A48801BalanceAnalyticalID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'a48801_balance_analytical'


class A48802BalanceAnalyticalID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'a48802_balance_analytical'


class P48809BalanceAnalyticalID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'p48809_balance_analytical'


class P47422BalanceAnalyticalID(ID):
    _service = SERVICE
    _app = APP
    _view_set = 'p47422_balance_analytical'


class A48801BalanceAnalyticalPoint(RetrievePointMixin, ContractObjectPoint):
    _point_id = A48801BalanceAnalyticalID()
    _read_contract = A48801BalanceAnalyticalReadContract


class A48802BalanceAnalyticalPoint(RetrievePointMixin, ContractObjectPoint):
    _point_id = A48802BalanceAnalyticalID()
    _read_contract = A48802BalanceAnalyticalReadContract


class P48809BalanceAnalyticalPoint(RetrievePointMixin, ContractObjectPoint):
    _point_id = P48809BalanceAnalyticalID()
    _read_contract = P48809BalanceAnalyticalReadContract


class P47422BalanceAnalyticalObjectPoint(RetrievePointMixin, ContractObjectPoint):
    _point_id = P47422BalanceAnalyticalID()
    _read_contract = P47422BalanceAnalyticalReadContract
